package br.unirio.processo.general.metaheuristics;

import br.unirio.processo.general.model.component.RepositorioComponentes;
import br.unirio.processo.general.model.process.Processo;

public class ProcessForcaBruta
{
	private ProcessCalculator calculator;

	public ProcessForcaBruta(RepositorioComponentes repositorio, Processo processo)
	{
		this.calculator = new ProcessCalculator(repositorio, processo);
	}

	public int[] execute()
	{
		int tamanhoSolucao = calculator.getSolutionSize();
		double numeroTotalCiclos = Math.pow(2, tamanhoSolucao);
		
		long initTime = System.currentTimeMillis();
		int[] currentSolution = new int[tamanhoSolucao];
		
		double bestFitness = 0.0;
		int[] bestSolution = new int[tamanhoSolucao];

		for (int ciclo = 0; ciclo < numeroTotalCiclos; ciclo++)
		{
			for (int i = 0; i < tamanhoSolucao; i++)
				currentSolution[i] = ((ciclo & (1 << i)) != 0) ? 1 : 0;
			
			double currentFitness = calculator.calculaFitness(currentSolution);
			
			if (currentFitness > bestFitness)
			{
				bestFitness = currentFitness;
				calculator.copySolution(currentSolution, bestSolution);
			}
		}
		
		long executionTime = System.currentTimeMillis() - initTime;
		System.out.println(calculator.imprimeSolucoes(bestSolution, 0, executionTime, bestFitness));
		return bestSolution;
	}
}