package br.unirio.processo.general.metaheuristics;

import br.unirio.processo.general.model.component.RepositorioComponentes;
import br.unirio.processo.general.model.process.Processo;

public class ProcessRandomSearch
{
	private ProcessCalculator calculator;

	public ProcessRandomSearch(RepositorioComponentes repositorio, Processo processo)
	{
		this.calculator = new ProcessCalculator(repositorio, processo);
	}
	
	public int[] execute(int ciclo)
	{
		int evaluations = 0;
		long initTime = System.currentTimeMillis();

		int[] bestSolution = calculator.geraSolucaoAleatoria();
		double bestFitness = calculator.calculaFitness(bestSolution);
		
		int tamanhoSolucao = calculator.getSolutionSize();
		int maxEvaluations = tamanhoSolucao * tamanhoSolucao;

		while (evaluations < maxEvaluations)
		{	
			int[] currentSolution = calculator.geraSolucaoAleatoria();
			double currentFitness = calculator.calculaFitness(currentSolution);
			
			if (currentFitness > bestFitness)
			{
				bestFitness = currentFitness;
				calculator.copySolution(currentSolution, bestSolution);
			}
			
			evaluations++;
		}
				
		long executionTime = System.currentTimeMillis() - initTime;
		System.out.println(calculator.imprimeSolucoes(bestSolution, ciclo, executionTime, bestFitness));
		return bestSolution;
	}
}