package br.unirio.processo.general.metaheuristics.genetic;

import jmetal.base.Algorithm;
import jmetal.base.Operator;
import jmetal.base.Solution;
import jmetal.base.SolutionSet;
import jmetal.base.operator.crossover.SinglePointCrossover;
import jmetal.base.operator.mutation.BitFlipMutation;
import jmetal.base.operator.selection.BinaryTournament;
import jmetal.metaheuristics.singleObjective.geneticAlgorithm.gGA;
import unirio.experiments.monoobjective.execution.MonoExperiment;
import br.unirio.processo.general.model.ProcessoCompleto;

public class ExperimentoAlgoritmoGenetico extends MonoExperiment<ProcessoCompleto>
{
	@Override
	protected Solution runCycle(ProcessoCompleto instance, int instanceNumber) throws Exception
	{
		ProcessProblem problem = new ProcessProblem(instance);
		int numeroComponentes = problem.pegaNumeroComponentesOtimizacao();

		int tamanhoPopulacao = numeroComponentes + ((numeroComponentes % 2 == 1) ? 1 : 0);
		int maxEvaluations = tamanhoPopulacao * tamanhoPopulacao;

		Operator crossover = new SinglePointCrossover();
		crossover.setParameter("probability", 0.8);

		Operator mutation = new BitFlipMutation();
		mutation.setParameter("probability", 0.02);

		Operator selection = new BinaryTournament();
				
		Algorithm ga = new gGA(problem);	
		ga.setInputParameter("populationSize", tamanhoPopulacao);
		ga.setInputParameter("maxEvaluations", maxEvaluations);
		ga.addOperator("crossover", crossover);
		ga.addOperator("mutation", mutation);
		ga.addOperator("selection", selection);
		
		SolutionSet solutions = ga.execute();
		return solutions.get(0);
	}
}