package br.unirio.processo.general.metaheuristics.genetic;

import jmetal.base.Problem;
import jmetal.base.Solution;
import jmetal.base.solutionType.BinarySolutionType;
import jmetal.base.variable.Binary;
import jmetal.util.JMException;
import br.unirio.processo.general.metaheuristics.ProcessCalculator;
import br.unirio.processo.general.model.ProcessoCompleto;

/**
 * Classe que representa um problema de otimizao de processo
 */
public class ProcessProblem extends Problem
{
	private ProcessCalculator calculator;
	private int evaluations;

	/**
	 * Prepara o problema de otimizao dos portfolios
	 */
	public ProcessProblem(ProcessoCompleto processoCompleto) throws ClassNotFoundException
	{
		this.calculator = new ProcessCalculator(processoCompleto.getRepositorio(), processoCompleto.getProcesso());
		this.evaluations = 0;
	
		numberOfObjectives_ = 1;
		numberOfVariables_ = 1;
		solutionType_ = new BinarySolutionType(this);
		length_ = new int[numberOfVariables_];
		length_[0] = calculator.getSolutionSize();
	}
	
	/**
	 * Avalia o portflio de acordo com os critrios da tcnica estudada.
	 */
	@Override
	public void evaluate(Solution solution) throws JMException
	{
		Binary portfolio = (Binary) solution.getDecisionVariables()[0];
		int[] components = new int[calculator.getSolutionSize()];
		
		for (int i = 0; i < components.length; i++)
			components[i] = portfolio.bits_.get(i) ? 1 : 0;
		
		double fitness = calculator.calculaFitness(components);
		solution.setObjective(0, -fitness);
		solution.setLocation(evaluations);
		evaluations++;
	}

	/**
	 * Retorna o nmero de componentes no problema
	 */
	public int pegaNumeroComponentesOtimizacao()
	{
		return calculator.getSolutionSize();
	}
}