package br.unirio.processo.general.model.component;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import br.unirio.processo.general.model.process.Artefato;

public class ComponenteProcesso
{
	private int id;
	private String nome;
	private String opcionalidade;
	private String variabilidade;
	private Float potencialColaboracao;
	private List<Artefato> artefatosEntradaMandatorios;
	private List<Artefato> artefatosEntradaOpcionais;
	private List<Artefato> artefatosSaida;
	private Boolean[] praticasColaboracao;
	
	public ComponenteProcesso(int id, String nome, String opcionalidade, String variabilidade, Float potencialColaboracao)
	{
		this.id = id;
		this.nome = nome;
		this.opcionalidade = opcionalidade;
		this.variabilidade = variabilidade;
		this.potencialColaboracao = potencialColaboracao;
		this.artefatosEntradaMandatorios = new ArrayList<Artefato>();
		this.artefatosEntradaOpcionais = new ArrayList<Artefato>();
		this.artefatosSaida = new ArrayList<Artefato>();
		this.praticasColaboracao = new Boolean[12];
	}

	public int getId()
	{
		return id;
	}

	public void setId(int id)
	{
		this.id = id;
	}

	public String getNome()
	{
		return nome;
	}

	public void setNome(String name)
	{
		this.nome = name;
	}

	public String getOpcionalidade()
	{
		return opcionalidade;
	}
	
	public void setOpcionalidade(String opcionalidade)
	{
		this.opcionalidade = opcionalidade;
	}

	public String getVariabilidade()
	{
		return variabilidade;
	}

	public void setVariabilidade(String variabilidade)
	{
		this.variabilidade = variabilidade;
	}

	/**
	 * Todos os artefatos de entrada (Mandatrios e Opcionais)
	 * @return
	 */
	public List<Artefato> getArtefatosEntrada()
	{
		List<Artefato> listaFull = new ArrayList<Artefato>(this.artefatosEntradaMandatorios);
		listaFull.addAll(artefatosEntradaOpcionais);
		return listaFull;
	}
			
	public Float getPotencialColaboracao()
	{
		return potencialColaboracao;
	}

	public void setPotencialColaboracao(Float potencialColaboracao)
	{
		this.potencialColaboracao = potencialColaboracao;
	}

	//artefatos entrada mandatrios
	public int getArtefatoEntradaMandatoriosCount()
	{
		return artefatosEntradaMandatorios.size();
	}
	
	public Artefato getArtefatoEntradaMandatorioIndex(int index)
	{
		return artefatosEntradaMandatorios.get(index);
	}
	
	public Artefato getArtefatoEntradaMandatorioPorNome(String nome)
	{
		for (Artefato artef : artefatosEntradaMandatorios)
			if (artef.getNome().compareToIgnoreCase(nome) == 0)
				return artef;
		
		return null;
	}
	
	public void addArtefatoEntradaMandatorio(Artefato item)
	{
		artefatosEntradaMandatorios.add(item);
	}
	
	public void removeArtefatoEntradaMandatorio(int index)
	{
		artefatosEntradaMandatorios.remove(index);
	}
	
	public Iterator<Artefato> getArtefatosEntradaMandatorios()
	{
		return artefatosEntradaMandatorios.iterator();
	}
	
	public List<Artefato> getListaArtefatosEntradaMandatorios()
	{
		return artefatosEntradaMandatorios;
	}
	
	public boolean checaArtefatoEntradaMandatorios(Artefato artef)
	{
		return artefatosEntradaMandatorios.contains(artef);
	}
	
	//artefatos entrada opcionais
	public int getArtefatoEntradaOpcionalCount()
	{
		return artefatosEntradaOpcionais.size();
	}
	
	public Artefato getArtefatoEntradaOpcionalIndex(int index)
	{
		return artefatosEntradaOpcionais.get(index);
	}
	
	public Artefato getArtefatoEntradaOpcionalPorNome(String nome)
	{
		for (Artefato artef : artefatosEntradaOpcionais)
			if (artef.getNome().compareToIgnoreCase(nome) == 0)
				return artef;
		
		return null;
	}
	
	public void addArtefatoEntradaOpcional(Artefato item)
	{
		artefatosEntradaOpcionais.add(item);
	}
	
	public void removeArtefatoEntradaOpcional(int index)
	{
		artefatosEntradaOpcionais.remove(index);
	}
	
	public Iterator<Artefato> getArtefatosEntradaOpcional()
	{
		return artefatosEntradaOpcionais.iterator();
	}
	
	public List<Artefato> getListaArtefatosEntradaOpcionais()
	{
		return artefatosEntradaOpcionais;
	}
	
	public boolean checaArtefatoEntradaOpcional(Artefato artef)
	{
		return artefatosEntradaOpcionais.contains(artef);
	}
	
	//todos
	public int getArtefatoEntradaTodosCount()
	{
		int total = 0;
		total = artefatosEntradaOpcionais.size();
		total += artefatosEntradaMandatorios.size();
		return total;
	}
	
	public boolean checaArtefatoEntradaTodos(Artefato artef)
	{
		boolean achou = false;
		if(artefatosEntradaMandatorios.contains(artef) || artefatosEntradaOpcionais.contains(artef))
			achou = true;
		return achou;
	}
	
	//artefatos sada
	public int getArtefatoSaidaCount()
	{
		return artefatosSaida.size();
	}
	
	public Artefato getArtefatoSaidaIndex(int index)
	{
		return artefatosSaida.get(index);
	}
	
	public Artefato getArtefatoSaidaPorNome(String nome)
	{
		for (Artefato artef : artefatosSaida)
			if (artef.getNome().compareToIgnoreCase(nome) == 0)
				return artef;
		
		return null;
	}
	
	public void addArtefatoSaida(Artefato item)
	{
		artefatosSaida.add(item);
	}
	
	public void removeArtefatoSaida(int index)
	{
		artefatosSaida.remove(index);
	}
	
	public Iterator<Artefato> getArtefatosSaida()
	{
		return artefatosSaida.iterator();
	}
	
	public boolean checaArtefatoSaida(Artefato artef)
	{
		return artefatosSaida.contains(artef);
	}
	
	//prticas de colaborao
	public Boolean[] getPraticasColaboracao() {
		return this.praticasColaboracao;
	}
	
	public void addPraticasColaboracao(Boolean[] praticasColaboracao) {
		this.praticasColaboracao = praticasColaboracao;
	}
	
	@Override
	public boolean equals(Object o)
	{
		ComponenteProcesso cp;
		try
		{
			cp = (ComponenteProcesso) o;
		}
		catch (ClassCastException erro)
		{
			return false;
		}
		if(this.id == cp.id && 
				this.nome == cp.nome && 
				this.opcionalidade == cp.opcionalidade && 
				this.variabilidade == cp.variabilidade && 
				this.potencialColaboracao == cp.potencialColaboracao)
		{
			return true;
		}
		return false;
	}
	
	public String toString(){
		return this.nome;
	}
}