package br.unirio.processo.general.model.component;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import br.unirio.processo.general.model.process.Artefato;
import br.unirio.processo.general.model.process.Processo;

public class RepositorioComponentes
{
	private List<ComponenteProcesso> componentes;
	private List<Artefato> artefatosGerados;

	public RepositorioComponentes()
	{
		this.componentes = new ArrayList<ComponenteProcesso>();
		this.artefatosGerados = new ArrayList<Artefato>();
	}

	// componentes
	public int getComponenteCount()
	{
		return componentes.size();
	}

	public ComponenteProcesso getComponenteIndex(int index)
	{
		return componentes.get(index);
	}

	public ComponenteProcesso getComponentePorNome(String nome)
	{
		for (ComponenteProcesso cp : componentes)
			if (cp.getNome().compareToIgnoreCase(nome) == 0)
				return cp;
		return null;
	}

	public void addComponente(ComponenteProcesso item)
	{
		componentes.add(item);
	}

	public void removeComponente(int index)
	{
		componentes.remove(index);
	}

	public Iterable<ComponenteProcesso> getComponentes()
	{
		return componentes;
	}

	// artefatos gerados
	public int getArtefatoGeradoCount()
	{
		return artefatosGerados.size();
	}

	public Artefato getArtefatoGeradoIndex(int index)
	{
		return artefatosGerados.get(index);
	}

	public void addArtefatoGerado(Artefato item)
	{
		if (!artefatosGerados.contains(item))
			artefatosGerados.add(item);
	}

	public Iterator<Artefato> getArtefatosGerados()
	{
		return artefatosGerados.iterator();
	}

	public boolean checaArtefatosGerados(Artefato artef)
	{
		return artefatosGerados.contains(artef);
	}

	public Vector<ComponenteProcesso> montaListaComponentesMandatorios()
	{
		Vector<ComponenteProcesso> cpmandatorios = new Vector<ComponenteProcesso>();
	
		for (ComponenteProcesso cp : componentes)
			if (cp.getOpcionalidade().equals("Mandatrio"))
				cpmandatorios.add(cp);

		return cpmandatorios;
	}

	// um componente  invivel, se ele tiver pelo menos 1 artefato de entrada
	// que no  satisfeito nem pela entrada do processo e nem pelos artefatos
	// gerados pelos outros componentes
	public Vector<ComponenteProcesso> montaListaComponentesInviaveis(Processo processo)
	{
		Vector<ComponenteProcesso> cpinviaveis = new Vector<ComponenteProcesso>();

		for (ComponenteProcesso cp : componentes)
		{
			for (Artefato artefato : cp.getListaArtefatosEntradaMandatorios())
			{
				if (!processo.checaEntradasDisponiveis(artefato))
				{
					if (!checaArtefatosGerados(artefato))
					{
						cpinviaveis.add(cp);
						break;
					}
				}
			}
		}
		return cpinviaveis;
	}

	public Vector<ComponenteProcesso> montaListaComponentesOtimizacao(Vector<ComponenteProcesso> cpmandatorios, Vector<ComponenteProcesso> cpinviaveis)
	{
		Vector<ComponenteProcesso> cpotimizacao = new Vector<ComponenteProcesso>();

		for (ComponenteProcesso cp : componentes)
			if (!cpmandatorios.contains(cp) && !cpinviaveis.contains(cp))
				cpotimizacao.add(cp);

		return cpotimizacao;
	}

	public int getIndiceComponente(ComponenteProcesso cp)
	{
		for (int i = 0; i < componentes.size(); i++)
			if (cp == componentes.get(i))
				return i;
		
		return -1;
	}
}