package br.unirio.processo.general.model.process;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class Processo {
	private String nome;
	private List<Artefato> entradasDisponiveis;
	private List<Artefato> produtosEsperados;
	
	public Processo(String nome){
		this.nome = nome;
		this.entradasDisponiveis = new ArrayList<Artefato>();
		this.produtosEsperados = new ArrayList<Artefato>();
	}
	
	public String getNome()
	{
		return nome;
	}

	public void setNome(String name)
	{
		this.nome = name;
	}
	
	//entradas disponveis
	public int getEntradaDisponivelCount()
	{
		return entradasDisponiveis.size();
	}
	
	public Artefato getEntradaDisponivelIndex(int index)
	{
		return entradasDisponiveis.get(index);
	}
	
	public Artefato getEntradaDisponivelPorNome(String nome)
	{
		for (Artefato artef : entradasDisponiveis)
			if (artef.getNome().compareToIgnoreCase(nome) == 0)
				return artef;
		
		return null;
	}
	
	public void addEntradaDisponivel(Artefato item)
	{
		entradasDisponiveis.add(item);
	}
	
	public void removeEntradaDisponivel(int index)
	{
		entradasDisponiveis.remove(index);
	}
	
	public Iterator<Artefato> getEntradasDisponiveis()
	{
		return entradasDisponiveis.iterator();
	}
	
	public boolean checaEntradasDisponiveis(Artefato artef)
	{
		return entradasDisponiveis.contains(artef);
	}
		
	//produtos esperados
	public int getProdutoEsperadoCount()
	{
		return produtosEsperados.size();
	}
	
	public Artefato getProdutoEsperadoIndex(int index)
	{
		return produtosEsperados.get(index);
	}
	
	public Artefato getProdutoEsperadoPorNome(String nome)
	{
		for (Artefato artef : produtosEsperados)
			if (artef.getNome().compareToIgnoreCase(nome) == 0)
				return artef;
		
		return null;
	}
	
	public void addProdutoEsperado(Artefato item)
	{
		produtosEsperados.add(item);
	}
	
	public void removeProdutoEsperado(int index)
	{
		produtosEsperados.remove(index);
	}
	
	public Iterator<Artefato> getProdutosEsperados()
	{
		return produtosEsperados.iterator();
	}
	
	public boolean checaProdutosEsperados(Artefato artef)
	{
		return produtosEsperados.contains(artef);
	}
		
	@Override
	public boolean equals(Object o)
	{
		Processo proc;
		try
		{
			proc = (Processo) o;
		}
		catch (ClassCastException erro)
		{
			return false;
		}
		if(this.nome == proc.nome)
		{
			return true;
		}
		return false;
	}
	
	public String toString(){
		return this.nome;
	}
}
