package br.unirio.processo.general.reader;

import java.util.Hashtable;

import unirio.experiments.instancegen.FlatPublisherReader;
import unirio.experiments.instancegen.FlatPublisherReaderException;
import br.unirio.processo.general.model.component.ComponenteProcesso;
import br.unirio.processo.general.model.component.RepositorioComponentes;
import br.unirio.processo.general.model.instance.Instance;
import br.unirio.processo.general.model.instance.Projeto;
import br.unirio.processo.general.model.process.Artefato;
import br.unirio.processo.general.model.process.LinhaProcesso;
import br.unirio.processo.general.model.process.Processo;

/**
 * Classe que faz a leitura da instncia contida no arquivo texto gerado 
 */
public class ModeloProcessoReader extends FlatPublisherReader 
{
	private RepositorioComponentes repositoriocomponentes;
	private LinhaProcesso linhaprocesso;
	private Instance instancia;
		
	public ModeloProcessoReader()
	{
		this.repositoriocomponentes = new RepositorioComponentes();
		this.linhaprocesso = new LinhaProcesso();
		this.instancia = new Instance();
	}
	
	public RepositorioComponentes getRepositorioComponentes()
	{
		return repositoriocomponentes;
	}
		
	public LinhaProcesso getLinhaProcesso()
	{
		return linhaprocesso;
	}
	
	public Instance getInstancia()
	{
		return instancia;
	}
	
	@Override
	protected void registerEntity(String type, String name, Hashtable<String, String> attributes) throws FlatPublisherReaderException 
	{
		if (type.compareToIgnoreCase("processo") == 0)
		{
			Processo proc = new Processo(name);
			linhaprocesso.addProcesso(proc);
			return;
		}
		
		if (type.compareToIgnoreCase("artefato") == 0)
		{
			String opcionalidade = attributes.get("opcionalidade");
			Artefato artef = new Artefato(name, opcionalidade);
			linhaprocesso.addArtefato(artef);
			return;
		}
		
		if (type.compareToIgnoreCase("componenteprocesso") == 0)
		{
			Integer id = Integer.parseInt(attributes.get("id"));
			String opcionalidade = attributes.get("opcionalidade");
			String variabilidade = attributes.get("variabilidade");
			Float potcolab = Float.parseFloat(attributes.get("potcolab"));
			ComponenteProcesso cp = new ComponenteProcesso(id, name, opcionalidade, variabilidade, potcolab);
			repositoriocomponentes.addComponente(cp);
			return;
		}
		
		if (type.compareToIgnoreCase("projeto") == 0)
		{
			Projeto proj = new Projeto(name);
			String algoritmo = attributes.get("algoritmo");
			String estrategia = attributes.get("estrategia");
			proj.setAlgoritmo(algoritmo);
			proj.setEstrategia(estrategia);
			instancia.addProjeto(proj);
			return;
		}
		
		generateException("entidade desconhecida '" + type + "'");
	}

	protected void registerRelationship(String type, String source, String target, Hashtable<String, String> attributes) throws FlatPublisherReaderException 
	{
		//entradas disponveis na linha de processo
		if (type.compareToIgnoreCase("entradadisponivel") == 0)
		{
			Processo proc = linhaprocesso.getProcessoPorNome(source);
			if (proc == null)
				generateException("processo desconhecido '" + source + "'");
			
			Artefato artef = linhaprocesso.getArtefatoPorNome(target);
			if (artef == null)
				generateException("artefato desconhecido '" + target + "'");

			proc.addEntradaDisponivel(artef);
			return;
		}
		
		//resultados esperados na linha de processo
		if (type.compareToIgnoreCase("resultadoesperado") == 0)
		{
			Processo proc = linhaprocesso.getProcessoPorNome(source);
			if (proc == null)
				generateException("processo desconhecido '" + source + "'");
			
			Artefato artef = linhaprocesso.getArtefatoPorNome(target);
			if (artef == null)
				generateException("artefato desconhecido '" + target + "'");

			proc.addProdutoEsperado(artef);
			return;
		}
		
		//lista de artefatos de entrada mandatrios do componente
		if (type.compareToIgnoreCase("requerObrigatoriamente") == 0)
		{
			ComponenteProcesso cp = repositoriocomponentes.getComponentePorNome(source);
			if (cp == null)
				generateException("componente desconhecido '" + source + "'");
			
			Artefato artef = linhaprocesso.getArtefatoPorNome(target);
			if (artef == null)
				generateException("artefato desconhecido '" + target + "'");

			cp.addArtefatoEntradaMandatorio(artef);
			return;
		}
		
		//lista de artefatos de entrada opcionais do componente
		if (type.compareToIgnoreCase("requerOpcionalmente") == 0)
		{
			ComponenteProcesso cp = repositoriocomponentes.getComponentePorNome(source);
			if (cp == null)
				generateException("componente desconhecido '" + source + "'");
			
			Artefato artef = linhaprocesso.getArtefatoPorNome(target);
			if (artef == null)
				generateException("artefato desconhecido '" + target + "'");

			cp.addArtefatoEntradaOpcional(artef);
			return;
		}
		
		//lista de artefatos de sada do componente
		if (type.compareToIgnoreCase("prov") == 0)
		{
			ComponenteProcesso cp = repositoriocomponentes.getComponentePorNome(source);
			if (cp == null)
				generateException("componente desconhecido '" + source + "'");
			
			Artefato artef = linhaprocesso.getArtefatoPorNome(target);
			if (artef == null)
				generateException("artefato desconhecido '" + target + "'");

			cp.addArtefatoSaida(artef);
			repositoriocomponentes.addArtefatoGerado(artef);
			return;
		}
		
		//lista de prticas de colaborao do componente
		if (type.compareToIgnoreCase("praticaColaboracao") == 0)
		{
			ComponenteProcesso cp = repositoriocomponentes.getComponentePorNome(source);
			if (cp == null)
				generateException("componente desconhecido '" + source + "'");
			
			String[] praticasColaboracaoTexto = target.split(",");
			Boolean[] praticasColaboracao = new Boolean[12];
			
			for(int i=0; i < praticasColaboracaoTexto.length; i++)
			{
				String pratica = praticasColaboracaoTexto[i];
				praticasColaboracao[i] = Boolean.valueOf(pratica.trim());
			}
			cp.addPraticasColaboracao(praticasColaboracao);

			return;
		}
		
		generateException("relacionamento desconhecido '" + type + "'");
	}
}