package br.unirio.processo.mono;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Vector;

import unirio.experiments.monoobjective.execution.StreamMonoExperimentListener;
import br.unirio.processo.general.metaheuristics.ProcessForcaBruta;
import br.unirio.processo.general.metaheuristics.ProcessHillClimbing;
import br.unirio.processo.general.metaheuristics.ProcessRandomSearch;
import br.unirio.processo.general.metaheuristics.genetic.ExperimentoAlgoritmoGenetico;
import br.unirio.processo.general.model.ProcessoCompleto;
import br.unirio.processo.general.model.component.ComponenteProcesso;
import br.unirio.processo.general.model.component.RepositorioComponentes;
import br.unirio.processo.general.model.instance.Instance;
import br.unirio.processo.general.model.instance.Projeto;
import br.unirio.processo.general.model.process.LinhaProcesso;
import br.unirio.processo.general.model.process.Processo;
import br.unirio.processo.general.reader.ModeloProcessoReader;

@SuppressWarnings("unused")
public class MainProgram
{
	private static final int CICLOS = 30;

	private static final boolean PARTIDA_VALIDA = true;
	private static final boolean PARTIDA_INVALIDA = false;

	private static final String ALGORITMO_HC = "HillClimbing";
	private static final String ALGORITMO_RS = "Random";
	private static final String ALGORITMO_FB = "ForaBruta";
	private static final String ALGORITMO_GA = "Genetico";

	public static void main(String[] args) throws Exception
	{
		// leitura dos dados
		ModeloProcessoReader reader = new ModeloProcessoReader();
		//reader.execute("data\\GAProcesso_Petro_ProjA-v10.txt");
		//reader.execute("data\\GAProcesso_Petro_ProjB-v16.txt");
		reader.execute("data\\GAProcesso_Petro_ProjC-v1.txt");

		// carrega os dados da instncia
		RepositorioComponentes repositorio = reader.getRepositorioComponentes();
		LinhaProcesso linha = reader.getLinhaProcesso();
		Processo processo = linha.getProcessoIndex(0);
		Instance instancia = reader.getInstancia();
		Projeto projeto = instancia.getProjetoIndex(0);

		// imprime a lista de prticas do componente
		arquivoPraticasColaboracaoComponente(projeto.getNome(), repositorio);

		// parmetros do algoritmo
		int tamanhoSolucao = repositorio.getComponenteCount();
		int maxEvaluations = tamanhoSolucao * tamanhoSolucao;

		System.out.println("\nGA");
		runGeneticAlgorithm(repositorio, processo);

		System.out.println("\nHC");
		runHillClimbing(repositorio, processo, PARTIDA_INVALIDA, maxEvaluations);

		System.out.println("\nRS");
		runRandomSearch(repositorio, processo, maxEvaluations);

		System.out.println("\nFB");
		runBruteForce(repositorio, processo, projeto.getNome());
	}

	private static void runGeneticAlgorithm(RepositorioComponentes repositorio, Processo processo) throws Exception
	{
		ProcessoCompleto pc = new ProcessoCompleto(repositorio, processo);
		Vector<ProcessoCompleto> vpc = new Vector<ProcessoCompleto>();
		vpc.add(pc);

		ExperimentoAlgoritmoGenetico ga = new ExperimentoAlgoritmoGenetico();
		ga.addListerner(new StreamMonoExperimentListener(new OutputStreamWriter(System.out), true));
		ga.run(vpc, CICLOS);
	}

	private static void runBruteForce(RepositorioComponentes repositorio, Processo processo, String nomeProjeto)
	{
		new ProcessForcaBruta(repositorio, processo).execute();
	}

	private static void runRandomSearch(RepositorioComponentes repositorio, Processo processo, int maxEvaluations)
	{
		ProcessRandomSearch prs = new ProcessRandomSearch(repositorio, processo);

		for (int i = 0; i < CICLOS; i++)
			prs.execute(i);
	}

	private static void runHillClimbing(RepositorioComponentes repositorio, Processo processo, boolean partidaValida, int maxEvaluations)
	{
		ProcessHillClimbing phc = new ProcessHillClimbing(repositorio, processo);

		for (int i = 0; i < CICLOS; i++)
			phc.execute(i, partidaValida);
	}

	/**
	 * Prepara o arquivo para salvar as prticas de colaborao
	 */
	private static BufferedWriter arquivoPraticasColaboracaoComponente(String projeto, RepositorioComponentes repositorio)
	{
		FileWriter fstream;
		BufferedWriter out = null;

		try
		{
			fstream = new FileWriter("result\\praticasColaboracao-" + projeto + ".txt");
			out = new BufferedWriter(fstream);
			out.write("Componente;Opcionalidade;PR1;PR2;PR3;PR4;PR5;PR6;PR7;PR8;PR9;PR10;PR11;PR12");

			for (ComponenteProcesso cp : repositorio.getComponentes())
			{
				out.write("\n" + cp.getNome() + ";");
				out.write(cp.getOpcionalidade() + ";");

				for (Boolean pratica : cp.getPraticasColaboracao())
					out.write(pratica.toString() + ";");
			}

			out.flush();
			out.close();

		} catch (IOException e)
		{
			e.printStackTrace();
		}
		return out;
	}
}